//******************************************************************************

// @File         model_vs.glsl

// @Version       1.0

// @Created      2017, 3, 8

// @Description  This file is model_vs.glsl.

// @HISTORY:

//******************************************************************************/

/* Macros in this shader:
_SKIN_							
*/

#include "common_vs.glsl"


attribute vec3		Position0;
attribute vec4		Normal0;// ע,XEngineдNormal0.z
attribute vec2		Texcoord0;
attribute vec2		Texcoord1;
attribute vec4		Binormal0;//ע⣬XEngineдʵֵTangent

uniform mat4		WorldViewProjMatrix;
uniform mat4		WorldMatrix;
uniform mat4		WorldViewMatrix;
uniform vec3		EyePos;
uniform vec3		LightDir;
uniform mat4		ViewMatrix;

varying vec2		VSOutTexcoord0;
varying vec2		VSOutTexcoord1;
varying vec3		VSOutNormal0;
varying vec3		VSOutViewDir;
varying vec3		VSOutLightDir;
varying vec3		VSOutTangent;
varying vec3		VSOutBinormal;
varying vec3		VSOutWorldViewDir;
varying mat3		VSOutInverseView;

#ifdef _SKIN_	
	#define MAX_MESH_BONE 30
	attribute vec4	BlendWeight0;
	attribute vec4	BlendIndices0;
    uniform vec4	BlendMatrix[MAX_MESH_BONE * 3];
#endif	


#ifdef _SKIN_
vec3 GetBlendPos( vec4 pos, int index, mediump float weight)
{
	vec3 posBlend;
	posBlend.x = dot(BlendMatrix[index], pos); 
	posBlend.y = dot(BlendMatrix[index + 1], pos); 
	posBlend.z = dot(BlendMatrix[index + 2], pos); 
    posBlend *= weight;
    return posBlend;
}
vec3 GetBlendNormal( vec3 normal, int index, mediump float weight)
{
	vec3 normalBlend;
	normalBlend.x = dot(BlendMatrix[index].xyz,  normal); 
	normalBlend.y = dot(BlendMatrix[index + 1].xyz,  normal); 
	normalBlend.z = dot(BlendMatrix[index + 2].xyz,  normal); 
    normalBlend *= weight;
    return normalBlend;
}

#endif

void main(void)
{
	 vec4 pos = vec4(Position0, 1.0);

#if defined(_SKIN_)

	float weightLast = 1.0 - BlendWeight0.x - BlendWeight0.y - BlendWeight0.z;

	ivec4 index = ivec4(BlendIndices0 * 3.0);
	
	 vec3 posBlend = 
	    GetBlendPos(pos, index.x, BlendWeight0.x) +
	    GetBlendPos(pos, index.y, BlendWeight0.y) +
	    GetBlendPos(pos, index.z, BlendWeight0.z) +
	    GetBlendPos(pos, index.w, weightLast);
    pos.xyz = posBlend;

	 vec3 normal = 
	    GetBlendNormal(Normal0.xyz, index.x, BlendWeight0.x) +
	    GetBlendNormal(Normal0.xyz, index.y, BlendWeight0.y) +
	    GetBlendNormal(Normal0.xyz, index.z, BlendWeight0.z) +
	    GetBlendNormal(Normal0.xyz, index.w, weightLast);


	 vec3 binormal = 
		GetBlendNormal(Binormal0.xyz, index.x, BlendWeight0.x) +
		GetBlendNormal(Binormal0.xyz, index.y, BlendWeight0.y) +
		GetBlendNormal(Binormal0.xyz, index.z, BlendWeight0.z) +
		GetBlendNormal(Binormal0.xyz, index.w, weightLast);

#else // #ifdef _SKIN_

	vec3 normal = Normal0.xyz;
	vec3 binormal = Binormal0.xyz;

#endif // #ifdef _SKIN_

	vec3 worldPos = (WorldMatrix * pos).xyz;
	gl_Position = WorldViewProjMatrix * pos;
	VSOutTexcoord0 = Texcoord0;
	VSOutTexcoord1 = Texcoord1;
	VSOutWorldViewDir = normalize(EyePos - worldPos);
	VSOutLightDir = LightDir;
	VSOutViewDir = (ViewMatrix * vec4(VSOutWorldViewDir.xyz, 0.0)).xyz;
	VSOutLightDir = (ViewMatrix * vec4(VSOutLightDir.xyz, 0.0)).xyz;

	// To write "mat3(WorldMatrix)" will make the compiler of PowerVR GLES simulator reporting warning C7536. So we cast the vector instead. Hope it will be optimized.
	VSOutNormal0 = (WorldViewMatrix * vec4(normal, 0.0)).xyz;

	VSOutBinormal = (WorldViewMatrix * vec4(binormal.xyz, 0.0)).xyz;

	// 2017.04.06 ޸
	//VSOutTangent = cross(VSOutBinormal, VSOutNormal0) * Binormal0.w;
	VSOutTangent = cross(VSOutBinormal, VSOutNormal0) * Normal0.w;

	// To write "mat3( VSOutBinormal, VSOutTangent, VSOutNormal0 )" will make the compiler of PowerVR GLES simulator reporting warning C7536.
	// So we dot the 3 components of vector manually instead of multipling it with the matrix.
	vec3 viewDir;
	viewDir.x = dot(VSOutViewDir, VSOutTangent);	// The tangent here is actually binormal
	viewDir.y = dot(VSOutViewDir, VSOutBinormal);	// The binormal here is actually tangent
	viewDir.z = dot(VSOutViewDir, VSOutNormal0);
	VSOutViewDir = viewDir;

	vec3 lightDir;
	lightDir.x = dot(VSOutLightDir, VSOutTangent);	// The tangent here is actually binormal
	lightDir.y = dot(VSOutLightDir, VSOutBinormal);	// The binormal here is actually tangent
	lightDir.z = dot(VSOutLightDir, VSOutNormal0);
	VSOutLightDir = lightDir;	
	
#ifdef REFLECT_ENABLE
	VSOutInverseView[0] = ViewMatrix[0].xyz;
	VSOutInverseView[1] = ViewMatrix[1].xyz;
	VSOutInverseView[2] = ViewMatrix[2].xyz;
	VSOutInverseView = Transpose_mat3(VSOutInverseView); // rotate inverse = transpose
#endif // #if REFLECT_ENABLE
}

